#!/bin/bash
#
# nohup ./data_transfer.sh >& expid.log &
#

##################################
####  User Defined Variables  #### 
##################################

dst1=/cfunas/exp/ecearth # store logfiles, MM*, REST*
dst2=/cfunas/exp/ecearth/gribfiles # store ICM*

plt= # choose platform (ithaca, mn, cfs-aux-4)
expid= # supply expid
sdates="" # supply list of start dates
members="" # supply list of members

remove=TRUE # remove output from source
if [[ $remove == TRUE ]]; then
 RSYNC="rsync -acv --remove-sent-files"
else
 RSYNC="rsync -acv"
fi
#####  End of User Defined Dariables  ####

#################################
####  User Defined Funtions  #### 
#################################

function transfer1(){
 dst=$1/$expid/$s/$m/$2
 for l in $list; do
  echo "tranfering file ... ${plt}:$l $dst"
  $RSYNC ${plt}:${l} $dst
  echo "done."
  echo
  echo
 done
}

function transfer2(){
 dst=$1/$expid/$s/$m/$2
 ssh $plt mkdir -p $tmp
 for l in $list; do
  echo "tranfering file ... ${hsm}:$l $tmp"
  ssh $plt rsync -acv ${hsm}:${l} $tmp
  nl=`basename $l`
  echo "tranfering file ... ${plt}:${tmp}/$nl $dst"
  rsync -acv --remove-sent-files ${plt}:${tmp}/$nl $dst
  if [[ $? -eq 0 && $remove == TRUE ]]; then
   $sshplt rm -f $l
  fi
  echo "done."
  echo
  echo
 done
 ssh $plt rm -rf $tmp
}
####  End of the User Defined Functions  #### 

###################################
####  Main Part of the Script  ####
###################################

date
echo "platform is: $plt"
echo "expid is: $expid"
echo "start dates: $sdates"
echo "members per start date: $members"

for s in $sdates; do
 for m in $members; do

  sshplt="ssh $plt"
  transfer="transfer1"

  case $plt in
   ithaca)
    src=/share/data/cfu/exp
   ;;
   mn)
    hsm="login8"
    sshplt="ssh $plt ssh $hsm"
    src=/HSM/ecm86/exp
    username=`ssh $plt env | grep -w USER | cut -d '=' -f2`
    tmp=/gpfs/scratch/ecm86/$username/${expid}_data_transfer
    transfer="transfer2"
   ;;
   cfs-aux-4)
    src=/cfs/klemming/nobackup/a/asifsami/exp
   ;;
   *)
    echo "$plt is not a valid machine"
   ;;
  esac

  #logfiles*
  lswc=`$sshplt ls -1 $src/$expid/$s/$m/logfiles* | wc -l`
  if [[ $lswc -gt 0 ]]; then
   mkdir -p $dst1/$expid/$s/$m
   list=`$sshplt ls -1 $src/$expid/$s/$m/logfiles*`
   $transfer $dst1 ""
  fi

  #MM*
  lswc=`$sshplt ls -1 $src/$expid/$s/$m/outputs/MM?_* | wc -l`
  if [[ $lswc -gt 0 ]]; then
   mkdir -p $dst1/$expid/$s/$m/outputs
   list=`$sshplt ls -1 $src/$expid/$s/$m/outputs/MM?_*`
   $transfer $dst1 outputs
  fi

  #REST*
  lswc=`$sshplt ls -1 $src/$expid/$s/$m/restarts/REST?_* | wc -l`
  if [[ $lswc -gt 0 ]]; then
   mkdir -p $dst1/$expid/$s/$m/restarts
   list=`$sshplt ls -1 $src/$expid/$s/$m/restarts/REST?_*`
   $transfer $dst1 restarts
  fi

  #ICM*
  lswc=`$sshplt ls -1 $src/$expid/$s/$m/outputs/ICM* | wc -l`
  if [[ $lswc -gt 0 ]]; then
   mkdir -p $dst2/$expid/$s/$m/outputs
   list=`$sshplt ls -1 $src/$expid/$s/$m/outputs/ICM*`
   $transfer $dst2 outputs
  fi

 done
done
date
####  End of the Main Part of Script  ####
